/*
	Eclipse Projekt Demonstration_SDP_Ampelsteuerung

	Demonstriert das Singleton Design Pattern (SDP) anhand einer
	Ampelsteuerung in der Programmiersprache C++. Die Zustnde der
	Ampel sind Rot, Gelb und Grn. Allerdings gibt es zwei Zustnde
	fr Gelb, je nach Richtung (Rot/Grn oder Grn/Rot).

	Autor: Winfried Trmper <pub+cpp@wt.tuxomania.net>
	Copyright (c) 2018 Winfried Trmper.
	Lizenz: CC BY (Creative Commons mit Namensnennung)

	Ausgabe des Programmlaufs:

+-R-+-Y-+-G-+
|###|   |   |
|###|   |   | Waiting... 4321
+-R-+-Y-+-G-+
|   |>>>|   |
|   |>>>|   | Waiting... 21
+-R-+-Y-+-G-+
|   |   |###|
|   |   |###| Waiting... 87654321
+-R-+-Y-+-G-+
|   |<<<|   |
|   |<<<|   | Waiting... 654321
+-R-+-Y-+-G-+
|###|   |   |
|###|   |   | Waiting... 4321
+-R-+-Y-+-G-+

	Dieses Projekt entstand als Lernhilfe fr einen Freund. Es ist wirklich
	nur eine Demonstration des SDP und keine besonders berzeugende Ampelsteuerung.
	Ich persnlich rate vom SDP als Standard-Lsung ab.

	Eine exemplarische Beschreibung einer Ampelphasen ist in Red.cpp enthalten.
 */

#include <iostream>
#include <unistd.h>

#include "Controller.h"
#include "Timer.h"
#include "State/Red.h"
#include "State/YellowFromRed.h"
#include "State/Green.h"
#include "State/YellowFromGreen.h"

/* --------------------------------------------------------------------- */

int main()
{
	Timer timer;
	Controller controller(&timer);

	/* Dauer jeder Ampelphase */
	State::Red            ::getInstance()->setDuration(3);
	State::YellowFromRed  ::getInstance()->setDuration(1);
	State::YellowFromGreen::getInstance()->setDuration(5);
	State::Green          ::getInstance()->setDuration(7);

	/* Erste Ampelphase */
	controller.setState(State::Red::getInstance());
	while(true)
	{
		controller.step();
		sleep(1);
		timer.step();
	}
	return(0);
}

/* --------------------------------------------------------------------- */

/* bersetzen unter Windows:
	oben #include "Windows.h"
	unten Sleep(1000);
 */
